from leapp.actors import Actor
from leapp.dialogs import Dialog
from leapp.dialogs.components import BooleanComponent
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api
from leapp import reporting
from leapp.reporting import Report, create_report
import subprocess
import os
import sys


class CheckRDMARepo(Actor):
    """
    Check if ol7_UEKR5_RDMA yum repo is enabled and inhibit the upgrade process.

    """

    name = 'check_rdma_repo'
    consumes = ()
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)
    dialogs = (
        Dialog(
            scope='confirm_RDMA_UEKR6_only_on_OL8',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Confirm RDMA UEKR6 on OL8 only? '
                          'If no, the upgrade process will be interrupted.',
                    description='Confirm only RDMA UEKR6 is available on OL8.',
                    reason='Only RDMA UEKR6 is available on OL8. '
                           'All previous versions of RDMA are unavailable.'
                ),
            )
        ),
    )

    def process(self):
        try:
            ps = subprocess.Popen("yum repolist", shell=True, stdout=subprocess.PIPE)
            output = ps.stdout.read()
            ps.stdout.close()
            ps.wait()

            if (output.find('ol7_UEKR5_RDMA') != -1):
                api.current_logger().info('Yum repo ol7_UEKR5_RDMA is enabled.')
                result = self.confirm()
                if result is True:
                    self.produce_report()
                else:
                    self.produce_inhibitor()
        except:
            api.current_logger().error('Unable to determine if ol7_UEKR5_RDMA is enabled.')
            raise StopActorExecutionError('Unable to determine if ol7_UEKR5_RDMA is enabled.')

    def confirm(self):
        answer = self.get_answers(self.dialogs[0])
        return answer.get('confirm')

    def produce_report(self):
        create_report([
            reporting.Title('Only RDMA UEKR6 is available on OL8.'),
            reporting.Summary(
                'Detected ol7_UEKR5_RDMA repo is enabled, confirmed that RDMA UEKR6 is only available in OL8'
            ),
            reporting.ExternalLink(
                title='Unbreakable Enterprise Kernel Release 6',
                url='https://docs.oracle.com/en/operating-systems/uek/6/relnotes6.2/ol_instav.html#uek6_install_rdma'
            ),
            reporting.Severity(reporting.Severity.HIGH)
        ])

    def produce_inhibitor(self):
        create_report([
            reporting.Title('Confirm only RDMA UEKR6 is available on OL8.'),
            reporting.Summary(
                'Upgrade process was interrupted because yum repo ol7_UEKR5_RDMA is enabled '
                'and need to confirm only RDMA UEKR6 is available on OL8.'
            ),
            reporting.ExternalLink(
                title='Unbreakable Enterprise Kernel Release 6',
                url='https://docs.oracle.com/en/operating-systems/uek/6/relnotes6.2/ol_instav.html#uek6_install_rdma'
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Tags([
                    reporting.Tags.SECURITY,
                    reporting.Tags.TOOLS
            ]),
            reporting.Flags([reporting.Flags.INHIBITOR]),
            reporting.Remediation(hint='Confirm only RDMA UEKR6 is available on OL8.')
        ])

