from leapp.actors import Actor
from leapp.libraries.actor.checkkerneldrivers import (
    check_drivers,
    get_present_drivers,
    get_removed_drivers,
)
from leapp.libraries import stdlib
from leapp.libraries.stdlib import api
from leapp.exceptions import StopActorExecution, StopActorExecutionError
from leapp.models import ActiveKernelModulesFacts
from leapp.reporting import Report, create_report
from leapp import reporting
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckKernelDrivers(Actor):
    """
    Actor checks if any loaded OL7 kernel driver is missing in OL8.
    If yes, the upgrade process will be inhibited.

    Inhibition is done because missing kernel driver on OL8 system may
    mean that the hardware using such driver would not work on OL8.

    Note:
     - List of kernel drivers missing on the OL8 system is located in the
        files/removed_drivers.txt file.
    """
    name = 'check_kernel_drivers'
    consumes = (ActiveKernelModulesFacts,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        if self.is_boot_kernel_uek():
            removed_file = 'files/removed_drivers_uek.txt'
            conflicting = check_drivers(get_removed_drivers(removed_file), get_present_drivers())

            if conflicting:
                title = ('Detected loaded kernel drivers which have been removed '
                         'in UEK. Upgrade cannot proceed.')
                URL = ('https://docs.oracle.com/en/operating-systems/oracle-linux/8/relnotes8.0/ol8-issues-known.html#ol8-issues-29120478')
                summary = ('Support for the following '
                           'device drivers has been removed in UEKR6: \n     - {}'
                           '\nPlease see {} for details.'.format('\n     - '.join(conflicting), URL))
                remediation = ('Remediation can be disable detected kernel drivers in '
                               'order to proceed with the upgrade process using the rmmod or modprobe -r.')
                create_report([
                    reporting.Title(title),
                    reporting.Summary(summary),
                    reporting.Severity(reporting.Severity.HIGH),
                    reporting.Tags([reporting.Tags.KERNEL, reporting.Tags.DRIVERS]),
                    reporting.Flags([reporting.Flags.INHIBITOR]),
                    reporting.Remediation(hint=remediation)
                ] + [reporting.RelatedResource('kernel-driver', kd) for kd in conflicting])
            return

        removed_file = 'files/removed_drivers.txt'
        conflicting = check_drivers(get_removed_drivers(removed_file), get_present_drivers())

        if conflicting:
            title = ('Detected loaded kernel drivers which have been removed '
                     'in OL8. Upgrade cannot proceed.')
            URL = ('https://docs.oracle.com/en/operating-systems/oracle-linux/8/relnotes8.0/ol8-issues-known.html#ol8-issues-29120478')
            summary = ('Support for the following OL7 '
                       'device drivers has been removed in OL8: \n     - {}'
                       '\nPlease see {} for details.'.format('\n     - '.join(conflicting), URL))
            remediation = ('Remediation can be use UEK6 or disable detected kernel drivers in '
                           'order to proceed with the upgrade process using the rmmod or modprobe -r.')
            create_report([
                reporting.Title(title),
                reporting.Summary(summary),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Tags([reporting.Tags.KERNEL, reporting.Tags.DRIVERS]),
                reporting.Flags([reporting.Flags.INHIBITOR]),
                reporting.Remediation(hint=remediation)
            ] + [reporting.RelatedResource('kernel-driver', kd) for kd in conflicting])

    def is_boot_kernel_uek(self):
        """
        Is boot kernel UEK or RHCK
        """
        try:
            current_default_kernel = stdlib.run(['grubby', '--default-kernel'])['stdout'].strip()
            api.current_logger().info('Detected default boot kernel {}'.format(current_default_kernel))
        except (OSError, stdlib.CalledProcessError):
            api.current_logger().error('Failed to query grubby for default kernel', exc_info=True)
            raise StopActorExecutionError('Unable to determine the current default boot kernel.')
            return

        if (current_default_kernel.find('uek') != -1):
            return True
        else:
            return False


